mt4_fun_num_map = {
"Client_Registration": 1
, "SignalBaseGetString": 2
, "StringToUpper": 3
, "iLowest": 4
, "ChartSaveTemplate": 5
, "TimeToStruct": 6
, "CheckPointer": 7
, "ExpertRemove": 8
, "OrderExpiration": 9
, "SeriesInfoInteger": 10
, "FileWriteDouble": 11
, "OnChartEvent": 12
, "ChartNavigate": 13
, "ceil": 14
, "iBars": 15
, "ChartNext": 16
, "IsStopped": 17
, "iAC": 18
, "SignalUnsubscribe": 19
, "iCustom": 20
, "StringCompare": 21
, "GlobalVariableName": 22
, "MathSrand": 23
, "ArrayFree": 24
, "GlobalVariablesDeleteAll": 25
, "FileReadString": 26
, "Interest_Arb_Action_Reply": 27
, "GetTickCount": 28
, "iLow": 29
, "TextSetFont": 30
, "round": 31
, "OrderComment": 32
, "SignalInfoGetInteger": 33
, "ChartSetInteger": 34
, "FileCopy": 35
, "ObjectCreate_002": 36
, "SetIndexEmptyValue": 37
, "SymbolInfoTick": 38
, "CopyTickVolume_003": 39
, "GlobalVariableSet": 40
, "OrderCloseBy": 41
, "ObjectCreate_001": 42
, "log": 43
, "IndicatorSetDouble": 44
, "WindowFind": 45
, "FileTell": 46
, "MathSqrt": 47
, "iMomentum": 48
, "WindowBarsPerChart": 49
, "FileWriteInteger": 50
, "WindowTimeOnDropped": 51
, "log10": 52
, "FileReadStruct": 53
, "FileIsLineEnding": 54
, "iStochastic": 55
, "CopyTickVolume_001": 56
, "AccountNumber": 57
, "WindowXOnDropped": 58
, "CryptEncode": 59
, "AccountInfoDouble": 60
, "SignalBaseTotal": 61
, "iMACD": 62
, "IndicatorSetInteger": 63
, "FileReadDatetime": 64
, "IndicatorDigits": 65
, "pow": 66
, "StringSubstr": 67
, "AccountLeverage": 68
, "TesterStatistics": 69
, "CopyTime_002": 70
, "MathLog": 71
, "iHigh": 72
, "CopyTime_003": 73
, "FileIsEnding": 74
, "TerminalClose": 75
, "AccountFreeMargin": 76
, "OrderPrint": 77
, "ObjectName": 78
, "iBandsOnArray": 79
, "ObjectSetFiboDescription": 80
, "ChartApplyTemplate": 81
, "Interest_Arb_Data": 82
, "CopyHigh_002": 83
, "FileFindClose": 84
, "ChartSetString": 85
, "ResourceSave": 86
, "OrderStopLoss": 87
, "SendMail": 88
, "cos": 89
, "OrderSelect": 90
, "rand": 91
, "Period": 92
, "iATR": 93
, "StringToCharArray": 94
, "ObjectMove": 95
, "FileIsExist": 96
, "CryptDecode": 97
, "SendNotification": 98
, "OrderOpenPrice": 99
, "SetIndexArrow": 100
, "iBands": 101
, "iMA": 102
, "ObjectGet": 103
, "acos": 104
, "MQLSetInteger": 105
, "StrToTime": 106
, "StrToDouble": 107
, "ColorToString": 108
, "TerminalPath": 109
, "TimeMinute": 110
, "SetLevelValue": 111
, "Interest_Arb_Account_Info": 112
, "ChartClose": 113
, "TimeDayOfWeek": 114
, "ObjectGetInteger": 115
, "tan": 116
, "ChartGetInteger": 117
, "WindowOnDropped": 118
, "ArrayResize": 119
, "TimeYear": 120
, "FileReadDouble": 121
, "iMAOnArray": 122
, "StringToColor": 123
, "iOBV": 124
, "ChartSymbol": 125
, "MessageBox": 126
, "sin": 127
, "TimeSeconds": 128
, "StringGetChar": 129
, "FileSize": 130
, "ChartIndicatorDelete": 131
, "ArraySort": 132
, "CopyTickVolume_002": 133
, "MathPow": 134
, "TimeToString": 135
, "SetIndexBuffer": 136
, "GlobalVariablesFlush": 137
, "iRSI": 138
, "iSAR": 139
, "ObjectGetFiboDescription": 140
, "MathSin": 141
, "GlobalVariableDel": 142
, "OrderSend": 143
, "MQLInfoInteger": 144
, "ArrayCopyRates": 145
, "DoubleToString": 146
, "Seconds": 147
, "iMFI": 148
, "ObjectGetDouble": 149
, "StringToInteger": 150
, "Sleep": 151
, "ChartTimePriceToXY": 152
, "ChartWindowFind": 153
, "fmod": 154
, "iADX": 155
, "iClose": 156
, "iVolume": 157
, "SetIndexShift": 158
, "ArraySetAsSeries": 159
, "StringLen": 160
, "ArrayCopySeries": 161
, "sqrt": 162
, "ResourceCreate": 163
, "Day": 164
, "iFractals": 165
, "EventSetMillisecondTimer": 166
, "ChartOpen": 167
, "ChartPriceOnDropped": 168
, "Comment": 169
, "iDeMarker": 170
, "ObjectSet": 171
, "ShortToString": 172
, "FolderDelete": 173
, "FileFindFirst": 174
, "TerminalInfoInteger": 175
, "ChartID": 176
, "asin": 177
, "SignalSubscribe": 178
, "OrderProfit": 179
, "MQLInfoString": 180
, "CharToStr": 181
, "WindowFirstVisibleBar": 182
, "PrintFormat": 183
, "SymbolName": 184
, "CopyOpen_002": 185
, "WindowRedraw": 186
, "ObjectsDeleteAll_001": 187
, "ChartGetString": 188
, "ArrayDimension": 189
, "AccountServer": 190
, "SymbolInfoSessionTrade": 191
, "CopyTime_001": 192
, "ChartYOnDropped": 193
, "FileWriteStruct": 194
, "iBearsPower": 195
, "TimeCurrent": 196
, "NormalizeDouble": 197
, "CopyClose_002": 198
, "ChartWindowOnDropped": 199
, "StringTrimRight": 200
, "Print": 201
, "WebRequest": 202
, "iAD": 203
, "AccountProfit": 204
, "MathTan": 205
, "iCCIOnArray": 206
, "CopyRates_002": 207
, "Minute": 208
, "MathMod": 209
, "IsTesting": 210
, "iOpen": 211
, "FileReadNumber": 212
, "CopyOpen_003": 213
, "StringSplit": 214
, "ObjectGetString": 215
, "StringConcatenate": 216
, "ShortArrayToString": 217
, "OrderTakeProfit": 218
, "iIchimoku": 219
, "AccountEquity": 220
, "MathIsValidNumber": 221
, "Symbol": 222
, "CopyHigh_001": 223
, "AccountStopoutMode": 224
, "CopyOpen_001": 225
, "ObjectGetTimeByValue": 226
, "CopyRates_001": 227
, "AccountInfoInteger": 228
, "StringAdd": 229
, "TerminalCompany": 230
, "ObjectDelete_002": 231
, "WindowPriceOnDropped": 232
, "GlobalVariablesTotal": 233
, "ObjectSetDouble_001": 234
, "AccountCurrency": 235
, "SymbolInfoSessionQuote": 236
, "StringReplace": 237
, "CopyClose_001": 238
, "WindowsTotal": 239
, "WindowScreenShot": 240
, "Interest_Arb_Order_Info": 241
, "ObjectGetShiftByValue": 242
, "ObjectFind_001": 243
, "DoubleToStr": 244
, "MathMax": 245
, "iWPR": 246
, "ChartRedraw": 247
, "ObjectsTotal_001": 248
, "AccountFreeMarginMode": 249
, "OnDeinit": 250
, "OnInit": 251
, "Month": 252
, "ArrayBsearch": 253
, "MathRound": 254
, "SignalInfoGetDouble": 255
, "OnTick": 256
, "StringGetCharacter": 257
, "FileWriteLong": 258
, "MathLog10": 259
, "ChartSetSymbolPeriod": 260
, "ArrayCompare": 261
, "ChartPeriod": 262
, "Interest_Arb_Swap_Info": 263
, "ObjectSetString_002": 264
, "EventKillTimer": 265
, "IsLibrariesAllowed": 266
, "WindowPriceMax": 267
, "CopyLow_003": 268
, "FileReadInteger": 269
, "AccountFreeMarginCheck": 270
, "FileWrite": 271
, "MarketInfo": 272
, "SendFTP": 273
, "Heartbeat": 274
, "FileFindNext": 275
, "SignalBaseGetInteger": 276
, "iEnvelopesOnArray": 277
, "WindowYOnDropped": 278
, "ObjectType": 279
, "Interest_Arb_Action": 280
, "ArrayRange": 281
, "AccountInfoString": 282
, "StructToTime": 283
, "FileWriteArray": 284
, "OrderOpenTime": 285
, "StringToLower": 286
, "GlobalVariableTime": 287
, "FileMove": 288
, "CharToString": 289
, "SymbolsTotal": 290
, "GlobalVariableGet": 291
, "ArrayIsDynamic": 292
, "SymbolSelect": 293
, "SignalInfoGetString": 294
, "OrderSymbol": 295
, "ObjectSetString_001": 296
, "ObjectSetDouble_002": 297
, "FileOpenHistory": 298
, "ObjectSetInteger_002": 299
, "StringBufferLen": 300
, "TerminalInfoDouble": 301
, "ChartXOnDropped": 302
, "SetIndexStyle": 303
, "TimeDayOfYear": 304
, "iStdDevOnArray": 305
, "iEnvelopes": 306
, "FileDelete": 307
, "iRSIOnArray": 308
, "iOsMA": 309
, "TimeDaylightSavings": 310
, "StrToInteger": 311
, "AccountMargin": 312
, "Hour": 313
, "SetIndexDrawBegin": 314
, "TimeMonth": 315
, "MathCos": 316
, "OrderDelete": 317
, "TimeLocal": 318
, "TerminalName": 319
, "ArrayGetAsSeries": 320
, "GetLastError": 321
, "IsOptimization": 322
, "Year": 323
, "FileOpen": 324
, "Bars_001": 325
, "HideTestIndicators": 326
, "StringSetChar": 327
, "OrderCommission": 328
, "GlobalVariableSetOnCondition": 329
, "fmin": 330
, "ObjectFind_002": 331
, "ColorToARGB": 332
, "GetPointer": 333
, "SymbolInfoInteger": 334
, "SymbolInfoDouble": 335
, "AccountName": 336
, "CopyLow_001": 337
, "StringFind": 338
, "FileGetInteger": 339
, "OrderClose": 340
, "TimeGMT": 341
, "AccountCredit": 342
, "ResourceFree": 343
, "IsExpertEnabled": 344
, "fabs": 345
, "ObjectsDeleteAll_003": 346
, "iBullsPower": 347
, "IsVisualMode": 348
, "OrderClosePrice": 349
, "StringInit": 350
, "ArrayInitialize": 351
, "ChartScreenShot": 352
, "TerminalInfoString": 353
, "FileReadArray": 354
, "ObjectSetInteger_001": 355
, "ObjectsTotal_002": 356
, "FileFlush": 357
, "PeriodSeconds": 358
, "AccountStopoutLevel": 359
, "StringFill": 360
, "TimeGMTOffset": 361
, "ChartIndicatorsTotal": 362
, "Digits": 363
, "iAlligator": 364
, "srand": 365
, "IsDemo": 366
, "OrderCloseTime": 367
, "iBWMFI": 368
, "OrderModify": 369
, "IsTradeContextBusy": 370
, "StringToShortArray": 371
, "ArrayCopy": 372
, "iGator": 373
, "EnumToString": 374
, "ChartSetDouble": 375
, "FolderCreate": 376
, "OrderSwap": 377
, "SymbolInfoString": 378
, "CopyRates_003": 379
, "IndicatorCounted": 380
, "MathExp": 381
, "TimeHour": 382
, "atan": 383
, "UninitializeReason": 384
, "ObjectDescription": 385
, "TextOut": 386
, "ChartTimeOnDropped": 387
, "PlaySound": 388
, "iMomentumOnArray": 389
, "WindowHandle": 390
, "FileClose": 391
, "OrderLots": 392
, "IntegerToString": 393
, "exp": 394
, "GlobalVariableTemp": 395
, "iAO": 396
, "SignalInfoSetDouble": 397
, "DebugBreak": 398
, "iHighest": 399
, "MathAbs": 400
, "OrderTicket": 401
, "iBarShift": 402
, "ObjectsDeleteAll_002": 403
, "MathArcsin": 404
, "fmax": 405
, "WindowExpertName": 406
, "ObjectSetText": 407
, "SignalBaseSelect": 408
, "MathArctan": 409
, "ChartGetDouble": 410
, "SignalInfoSetInteger": 411
, "TimeToStr": 412
, "MathFloor": 413
, "SetLevelStyle": 414
, "ResourceReadImage": 415
, "IsDllsAllowed": 416
, "GlobalVariableCheck": 417
, "WindowIsVisible": 418
, "CopyClose_003": 419
, "ObjectDelete_001": 420
, "DayOfYear": 421
, "ArraySize": 422
, "MathMin": 423
, "EventChartCustom": 424
, "Interest_Arb_Volatility_Info": 425
, "FolderClean": 426
, "IndicatorSetString": 427
, "ArrayIsSeries": 428
, "OnTester": 429
, "FileReadBool": 430
, "iForce": 431
, "CharArrayToString": 432
, "EventSetTimer": 433
, "RefreshRates": 434
, "ResetLastError": 435
, "ChartIndicatorName": 436
, "StringTrimLeft": 437
, "iStdDev": 438
, "OrderMagicNumber": 439
, "OrderType": 440
, "ZeroMemory": 441
, "ObjectGetValueByShift": 442
, "ChartFirst": 443
, "ObjectGetValueByTime": 444
, "StringToDouble": 445
, "iRVI": 446
, "Alert": 447
, "DayOfWeek": 448
, "FileWriteString": 449
, "IndicatorBuffers": 450
, "OnTimer": 451
, "WindowPriceMin": 452
, "IsTradeAllowed": 453
, "AccountBalance": 454
, "AccountCompany": 455
, "MathArccos": 456
, "FileSeek": 457
, "floor": 458
, "StringToTime": 459
, "FileWriteFloat": 460
, "StringFormat": 461
, "CopyHigh_003": 462
, "SignalBaseGetDouble": 463
, "OrdersTotal": 464
, "FileReadLong": 465
, "OrdersHistoryTotal": 466
, "StringSetCharacter": 467
, "Point": 468
, "FileReadFloat": 469
, "ChartXYToTimePrice": 470
, "IsConnected": 471
, "MathRand": 472
, "ArrayFill": 473
, "ArrayMinimum": 474
, "TextGetSize": 475
, "CopyLow_002": 476
, "iCCI": 477
, "TimeDay": 478
, "Bars_002": 479
, "IndicatorShortName": 480
, "ArrayMaximum": 481
, "SetIndexLabel": 482
, "MathCeil": 483
, "iTime": 484
}



from typing import Dict, List, Set, Tuple, Union, Any
from datetime import datetime
import logging
import time
import decimal

import pysailfish.internal.observability.log_helper as lh
from pysailfish.internal.DataCtrl.MT4DataCtrl.MT4DataCtrl_Server import MT4DataCtrl_Server
from pysailfish.internal.Network.TCPCast import TCPCast, TCPStatus

class MT4_EA_Server(object):
    def __init__(self):
        self.__data_ctrl_server: MT4DataCtrl_Server = None
        self.__logger: logging.Logger = None

    def InitComponent(self
                      , logger: logging.Logger
                      , data_ctrl_server: MT4DataCtrl_Server) -> None:
        self.__logger = logger
        self.__data_ctrl_server = data_ctrl_server

        return None

    def Start_EA(self) -> None:
        self.__data_ctrl_server.start_th()

        while True:
            time.sleep(1)

        return None

    def Process_MT4_Message(self, fun_name: str, parts: List[str]) -> None:
        self.__logger.info(parts)
        return None

    def Process_No_MT4_Message(self) -> None:
        time.sleep(0.01)

        return None


def main() -> None:
    ea_name: str = "MT4_Server_EA"
    logger = lh.init_logger(logger_name=f"{ea_name}_logger", is_json_output=False)

    ea = MT4_EA_Server()
    # initial tcp cast
    tcp_server: TCPCast = TCPCast()
    tcp_server.InitComponent(if_address="127.0.0.1"
                             , if_port=23456
                             , is_client=False
                             , num_clients=10
                             , timeout_in_seconds=0)
    # initial mt4 data control
    mt4_data_ctrl: MT4DataCtrl_Server = MT4DataCtrl_Server()
    mt4_data_ctrl.init_component(logger=logger
                                 , tcp_server=tcp_server
                                 , mt4_msg_callback=ea.Process_MT4_Message
                                 , mt4_no_msg_callback=ea.Process_No_MT4_Message)
    # initial EA as a server
    ea.InitComponent(logger=logger, data_ctrl_server=mt4_data_ctrl)
    ea.Start_EA()

if __name__ == "__main__":
    main()
